﻿DECLARE @DBMaxValueLength INT = 30;

DECLARE @ExceptionTableList TABLE
(
  TableName VARCHAR(MAX)
);

INSERT INTO @ExceptionTableList
  SELECT 'SysMsgLib';

DECLARE @SchemaColumn TABLE
(
  ShemaName					   VARCHAR(MAX),
  SchemaId					   UNIQUEIDENTIFIER,
  SchemaIdString			   VARCHAR(MAX) NULL,
  ColumnId					   UNIQUEIDENTIFIER,
  ColumnIdString			   VARCHAR(MAX) NULL,
  HashBytes					   AS HASHBYTES('SHA1', SchemaIdString + ColumnIdString),
  AfterInsertUpdateTriggerName VARCHAR(MAX),
  AfterDeleteTriggerName	   VARCHAR(MAX)
);

INSERT INTO @SchemaColumn (ShemaName, SchemaId, ColumnId)
  SELECT ss.Name,
		 CASE
		   WHEN ss.ExtendParent = 1 THEN ssParent.UId
		   ELSE ss.UId
		 END AS schemaId,
		 sesr.ColumnUId AS columnId
  FROM SysEntitySchemaReference sesr
  JOIN SysSchema ss ON ss.Id = sesr.SysSchemaId
  JOIN SysSchema ss1 ON ss1.Id = sesr.ReferenceSchemaId
	AND ss1.Name = 'SysImage'
  LEFT JOIN SysSchema AS ssParent ON ssParent.Id = ss.ParentId;

UPDATE @SchemaColumn
SET SchemaIdString = REPLACE(CONVERT(NVARCHAR(MAX), SchemaId), '-', ''),
	ColumnIdString = REPLACE(CONVERT(NVARCHAR(MAX), ColumnId), '-', '');

UPDATE @SchemaColumn
SET AfterInsertUpdateTriggerName = 'TRIL' + REPLACE(REPLACE(REPLACE(CAST('' AS XML).value('xs:base64Binary(sql:column("HashBytes"))', 'varchar(max)'), '=', ''), '/', ''), '+', '')

UPDATE @SchemaColumn
SET AfterInsertUpdateTriggerName = CASE
									WHEN LEN(AfterInsertUpdateTriggerName) > @DBMaxValueLength - 3 THEN LEFT(AfterInsertUpdateTriggerName, @DBMaxValueLength - 3)
									ELSE AfterInsertUpdateTriggerName
								  END + 'AIU',
	AfterDeleteTriggerName		 = CASE
									WHEN LEN(AfterInsertUpdateTriggerName) > @DBMaxValueLength - 2 THEN LEFT(AfterInsertUpdateTriggerName, @DBMaxValueLength - 2)
									ELSE AfterInsertUpdateTriggerName
								  END + 'AD';

DECLARE @sql NVARCHAR(MAX) = '';
DECLARE @flag BIT = 1;

SELECT @sql = COALESCE('DROP TRIGGER [dbo].[' + sysobjects.Name + '];' + @sql, sysobjects.Name)
FROM sysobjects
WHERE sysobjects.type = 'TR'
AND LEN(sysobjects.Name) = 30
AND LEFT(sysobjects.Name, 4) = 'TRIL'
AND NOT EXISTS (
  SELECT NULL
  FROM @SchemaColumn AS sc
  WHERE sc.AfterDeleteTriggerName = sysobjects.Name
  OR sc.AfterInsertUpdateTriggerName = sysobjects.Name
)
AND OBJECT_NAME(parent_obj) NOT IN (
  SELECT TableName
  FROM @ExceptionTableList
);

BEGIN TRANSACTION

BEGIN TRY
  EXEC (@sql);
END TRY
BEGIN CATCH
  IF @@trancount > 0
	ROLLBACK TRANSACTION;
END CATCH;

IF @@trancount > 0
  COMMIT TRANSACTION;
